# File formats for storing private keys for SSH

## Introduction

This is a two-part article on the file formats used for storing SSH
keys. This second part covers the file formats for storing SSH private
keys.  Part 1 covered file formats for storing public keys for SSH.

The Secure Shell (SSH) is a remote access protocol that supports the
use of public key cryptography to authenticate the client. In public
key cryptography, a credential consists of a key-pair: the private key
and its correspoinding public key. The SSH server is configured with
the public key (for example, with OpenSSH it can be stored in the
".ssh/authorized_keys" file in the user's home directory) and the
client must have the corresponding private key to login.

While the SSH protocol defines the wire-format for the data that is
transmitted between the client and server, it does not define how the
clients and servers store the private and public keys in files.
Therefore, different implementations of the SSH protocol have used
different file formats to store the keys.

This article describes some of the file formats that have been used to
store SSH private keys. It will cover the following file formats:

- OpenSSH Private Key
- PKCS #1 Private Key
- PuTTY Private Key (PPK)

This document only covers RSA keys, even though these file formats can
be used to store private keys for other cryptographic algorithms.


## OpenSSH Private Key

```
-----BEGIN OPENSSH PRIVATE KEY-----
b3BlbnNzaC1rZXktdjEAAAAABG5vbmUAAAAEbm9uZQAAAAAAAAABAAABFwAAAAdz
c2gtcnNhAAAAAwEAAQAAAQEA2MHejuGfkkzMU5RQj3DfFQKvWTSr2kEWG7BAyitM
nb08XcLe9sUaPLLqWF9b4A4EaWAYYA8vDjxDCw7pOtbF0XW0yExVtQffK1LL7yxG
BzOMCUJ5Fmb3Mmc+/Yndh96++aMPhC+kheM8a0pyfQMdPXCjdqDN50FJ9I5GqhuB
2/7YWymxfEs3rwa3CNC3eCuuDHbnNl0G1tqjIpE+MTwbbu28D7j9V4YqWTgXAgZ1
3a1gmZOqX2LQP2quRJ+2tiQMoJSKRLACG7UlZo+tgYLLZkjsv+LUI1Yp4eiBvMvI
saufRm5gvCj/c1EN4YS30/56q3l6XodN/XKLVE1iR8lTgQAAA9Al7g6HJe4OhwAA
AAdzc2gtcnNhAAABAQDYwd6O4Z+STMxTlFCPcN8VAq9ZNKvaQRYbsEDKK0ydvTxd
wt72xRo8supYX1vgDgRpYBhgDy8OPEMLDuk61sXRdbTITFW1B98rUsvvLEYHM4wJ
QnkWZvcyZz79id2H3r75ow+EL6SF4zxrSnJ9Ax09cKN2oM3nQUn0jkaqG4Hb/thb
KbF8SzevBrcI0Ld4K64Mduc2XQbW2qMikT4xPBtu7bwPuP1XhipZOBcCBnXdrWCZ
k6pfYtA/aq5En7a2JAyglIpEsAIbtSVmj62BgstmSOy/4tQjVinh6IG8y8ixq59G
bmC8KP9zUQ3hhLfT/nqreXpeh039cotUTWJHyVOBAAAAAwEAAQAAAQEAn3W4WhrH
/qP66T3CKlRZ+SX5g8HI745USlhWwaQrABxYDY4v4xDLpX3mAs9cUq9dNXxZUkm5
T1qr47W7hOq9qsNSFMx7fOtmlYOavcAvYFremW1KkV18W5sy7uh+jdcjXZlE7/Tw
rwKlzzR8l1U0LJ/x0XKpJ4iTdJM6jygfV4x/2HOuHxEXovoSUutxAejhbHqWiApr
hEI7JemFfRC0/HAyKsXbnv9tmnUH4aOtDsx2RwvCzv1LtGC8tobkm3RSRmrsT/o0
zjeS5QCO1AVqImIuYhdv8Ddbf/BvSANg3kQ+sGvnDO/o+PXtNqXkl15Vbi/bvQRK
A+sdzAviCJCOwQAAAIBnGMa/WL/8d0whd5SZVe7PSa6owdKFTFrB4Y/O7IZMin32
nDLJyqBeTWUev6AlWCo1XQ7HZgPUflE/Px2SeoBq+nZE3JmilHU/KTwoMysgNfB4
KO40VUtEgCfEHmLGtOsG3HTZHnEIcmIJwQsvJfRplmVJJVQ8dU9eq8B1TwuABQAA
AIEA+MFThA474/CVBFjao6/7l+LqRWwUVPfmNg7pQbRf8MlGNc3ekXgd3wtjUlVx
Nf1JVtcauR38f5ZVDL8Bu2H7emxn1GQFqku4lwVKOIXM2H0WPk2zBcqNMRfZS6Tr
QsLZPkzVJvxHVjMAaS2UpA5yY9PsuOYamAWcCGYA8PWk7ukAAACBAN8R+QRjAPiY
lOqGYxDm+Bv0DwNsQ5xDCOHph7znyYNZ8lttxStprye5TuLJoTrfSJjJSN+yh3Me
CQD6x4DiMftA8uubkmhgbnoKXSfGrkaCUG2R3wB0+9HYih2LN7tCxMfdPGRw4gcq
dIlYCSS1XQpYZ+PxLegbcHLTUOhiglDZAAAAEHVzZXJAZXhhbXBsZS5jb20BAgME
BQYHCAkK
-----END OPENSSH PRIVATE KEY-----
```

Other programs (e.g. PuTTYgen) refers to this file format as the "new
OpenSSH format".

## PKCS #1 Private Key


```
-----BEGIN RSA PRIVATE KEY-----
MIIEpQIBAAKCAQEA2MHejuGfkkzMU5RQj3DfFQKvWTSr2kEWG7BAyitMnb08XcLe
9sUaPLLqWF9b4A4EaWAYYA8vDjxDCw7pOtbF0XW0yExVtQffK1LL7yxGBzOMCUJ5
Fmb3Mmc+/Yndh96++aMPhC+kheM8a0pyfQMdPXCjdqDN50FJ9I5GqhuB2/7YWymx
fEs3rwa3CNC3eCuuDHbnNl0G1tqjIpE+MTwbbu28D7j9V4YqWTgXAgZ13a1gmZOq
X2LQP2quRJ+2tiQMoJSKRLACG7UlZo+tgYLLZkjsv+LUI1Yp4eiBvMvIsaufRm5g
vCj/c1EN4YS30/56q3l6XodN/XKLVE1iR8lTgQIDAQABAoIBAQCfdbhaGsf+o/rp
PcIqVFn5JfmDwcjvjlRKWFbBpCsAHFgNji/jEMulfeYCz1xSr101fFlSSblPWqvj
tbuE6r2qw1IUzHt862aVg5q9wC9gWt6ZbUqRXXxbmzLu6H6N1yNdmUTv9PCvAqXP
NHyXVTQsn/HRcqkniJN0kzqPKB9XjH/Yc64fERei+hJS63EB6OFsepaICmuEQjsl
6YV9ELT8cDIqxdue/22adQfho60OzHZHC8LO/Uu0YLy2huSbdFJGauxP+jTON5Ll
AI7UBWoiYi5iF2/wN1t/8G9IA2DeRD6wa+cM7+j49e02peSXXlVuL9u9BEoD6x3M
C+IIkI7BAoGBAPjBU4QOO+PwlQRY2qOv+5fi6kVsFFT35jYO6UG0X/DJRjXN3pF4
Hd8LY1JVcTX9SVbXGrkd/H+WVQy/Abth+3psZ9RkBapLuJcFSjiFzNh9Fj5NswXK
jTEX2Uuk60LC2T5M1Sb8R1YzAGktlKQOcmPT7LjmGpgFnAhmAPD1pO7pAoGBAN8R
+QRjAPiYlOqGYxDm+Bv0DwNsQ5xDCOHph7znyYNZ8lttxStprye5TuLJoTrfSJjJ
SN+yh3MeCQD6x4DiMftA8uubkmhgbnoKXSfGrkaCUG2R3wB0+9HYih2LN7tCxMfd
PGRw4gcqdIlYCSS1XQpYZ+PxLegbcHLTUOhiglDZAoGBALE566qWCY1ohoLGW19r
rjCUdpxnWEUBKX8yOHWbcQ84F1GWDxBNJ9sVXfn27eWtQ2cfceaZup+cF5QxMaaz
NFwQiBkF+hNOR4ExP6Ptj/hdhk91RCrIGLf1adc+9G5oLCb6zpIsAxci1xQYaWJI
+K6u+mTxa9kLhhNluR3zpUKhAoGBAI1qLG6OgRAYUte26FjPw9ycxWPLH7WRfbES
Rj4Ix2RhAlbZ6RRThHnvbUYywua6pKBPgsZlvJ7LHLQlR5K6UytQim+5CYDoGUF/
Dn1n5BXJCUndHv2ALCBlYXHHT0aE1pFJ/L5EHdajIIvtZqaB34DueLY1sH+j3Y69
zl30DV9JAoGAZxjGv1i//HdMIXeUmVXuz0muqMHShUxaweGPzuyGTIp99pwyycqg
Xk1lHr+gJVgqNV0Ox2YD1H5RPz8dknqAavp2RNyZopR1Pyk8KDMrIDXweCjuNFVL
RIAnxB5ixrTrBtx02R5xCHJiCcELLyX0aZZlSSVUPHVPXqvAdU8LgAU=
-----END RSA PRIVATE KEY-----
```

OpenSSH refers to this file format as "PEM". But that name is not very
precise, since PEM encoding can be used for public keys and other
types of binary data. To be more precise, this document refers to this
file format as the "RFC 7468 Textual Encoding of PKCS #1 Private Key",
or "PKCS #1 Private Key" for short.

Other programs (e.g. PuTTYgen) refers to this file format as the
"OpenSSH format" (as distinct from the "new OpenSSH format") since
this was the original/old file format used by OpenSSH.

## PuTTY Private Key (PPK)

For example,

```
PuTTY-User-Key-File-2: ssh-rsa
Encryption: none
Comment: someone@example.com
Public-Lines: 6
AAAAB3NzaC1yc2EAAAADAQABAAABAQDJnryCPo8+CBrQL7U1sjaj/4CsuaH3x+4o
offB8dKxJAw6J+DQQ044NQoNd1cNF6kgu27aaPC+evWaDjw0/dv4bGunyh3o3PWk
utPpvTUptLWWxKfxXmUQUrX5Pk5eXF6v5LQ9TNoeUbBdE92/QftSqQFR3nFwV2lm
9bQBFOTQLBGSscja6636caIQfV8lYgJyfg4IPZnNtMYEGxOzF9KZCO7pPsMh+4Js
8FHj1gqg9Ri1pFyZLNIlgDRtUA2qY+FD3FRC0ofBhI8V7VDVUoLzfHfedraizwsU
bMZVvStC2GSory8zmPMp+emCBOISUmvTwoeC+nHgDW3vKDaua1Ad
Private-Lines: 14
AAABAF9IgVYcMp3iPqm8oirqOiBvE2SNphnvhoH1aZ1ip2vH2W9ygTBrXn/5sPKE
P8OUNv2dFtppVbzvZzqTF6kDf/17X7VAM7plEkHzeUqxfHplSJwj/Cp5rdF1BULx
SyAVDzqZHwUJWNuTR4H1bYqBqEB8Vn9WXM32wX1DCPqp4Sjvk+80P9edf3+Y5IFC
VgQmbPUVM2GKzhBOe1FjroHnWwIWr/O7oZzBDZqzF4y0R0i8okqsCCDqSWkvM29A
8syTYEV6VyUxVdfXTiApg3wYdWhB0VWpXJ4qbN1L3jGFawlQHUM/5tj3O+swjMrt
LTd5Y/2ce8si9MU03Qkvl4RzEhEAAACBAOVzVG2+no81XG1hOo5t3AQe9QrY0Sr8
vhZSZ6Wh9cTtEASc187D5kYXxyGxs0vO0RdC0XT1cLizGrnv8RHsqHw5Jq0yEnJg
pzbAs2MZAJAESQRlQbWpP4rFPDEgD1hd0ZB+5CgeY1qP7x7cku+D6tdAScr3WhQt
RND9CWhIyI9zAAAAgQDg8wbfjvuQMmqBVn+wS6NX7cCNj4RfhA5+f6+pZnvooVhq
Mh8IiD+JI1SdAUV18YzjAQmy3VeSQp3YImn54IdII3OW3m0BoWzq5tW4ZKPMYMaY
7+ksbCDzEfxZUpyPjr3W4S3VQEWoL+A1LT9+VmitBGhs3myf4yxPVQEC7rCeLwAA
AIEA2lwJ3kfWf2HtBy93o+Pye9m1BHaGGr8RzpsfTHwlQv+4zw7HpY6P1l8gkgUV
yDNARZ0FYVsz9fs0iTacT3kqxYVvR4KlI1RbdOibb/k/pkzwbv4I2zn/UuIjP+TS
U2hkfvbMNLftTYn+6AxStBj52pMey3Ocdmfvp5RqUNoJqr4=
Private-MAC: 200f3733e416932e8f225d7a0a4212514a8a4879
```

The public lines contains base-64 encoded



## SSH.com Private Key

```
---- BEGIN SSH2 ENCRYPTED PRIVATE KEY ----
Comment: "user@example.com"
P2/56wAAA+4AAAA3aWYtbW9kbntzaWdue3JzYS1wa2NzMS1zaGExfSxlbmNyeXB0e3JzYS
1wa2NzMXYyLW9hZXB9fQAAAARub25lAAADnwAAA5sAAAARAQABAAAIAJ91uFoax/6j+uk9
wipUWfkl+YPByO+OVEpYVsGkKwAcWA2OL+MQy6V95gLPXFKvXTV8WVJJuU9aq+O1u4Tqva
rDUhTMe3zrZpWDmr3AL2Ba3pltSpFdfFubMu7ofo3XI12ZRO/08K8Cpc80fJdVNCyf8dFy
qSeIk3STOo8oH1eMf9hzrh8RF6L6ElLrcQHo4Wx6logKa4RCOyXphX0QtPxwMirF257/bZ
p1B+GjrQ7MdkcLws79S7RgvLaG5Jt0UkZq7E/6NM43kuUAjtQFaiJiLmIXb/A3W3/wb0gD
YN5EPrBr5wzv6Pj17Tal5JdeVW4v270ESgPrHcwL4giQjsEAAAgA2MHejuGfkkzMU5RQj3
DfFQKvWTSr2kEWG7BAyitMnb08XcLe9sUaPLLqWF9b4A4EaWAYYA8vDjxDCw7pOtbF0XW0
yExVtQffK1LL7yxGBzOMCUJ5Fmb3Mmc+/Yndh96++aMPhC+kheM8a0pyfQMdPXCjdqDN50
FJ9I5GqhuB2/7YWymxfEs3rwa3CNC3eCuuDHbnNl0G1tqjIpE+MTwbbu28D7j9V4YqWTgX
AgZ13a1gmZOqX2LQP2quRJ+2tiQMoJSKRLACG7UlZo+tgYLLZkjsv+LUI1Yp4eiBvMvIsa
ufRm5gvCj/c1EN4YS30/56q3l6XodN/XKLVE1iR8lTgQAAA/9nGMa/WL/8d0whd5SZVe7P
Sa6owdKFTFrB4Y/O7IZMin32nDLJyqBeTWUev6AlWCo1XQ7HZgPUflE/Px2SeoBq+nZE3J
milHU/KTwoMysgNfB4KO40VUtEgCfEHmLGtOsG3HTZHnEIcmIJwQsvJfRplmVJJVQ8dU9e
q8B1TwuABQAABADfEfkEYwD4mJTqhmMQ5vgb9A8DbEOcQwjh6Ye858mDWfJbbcUraa8nuU
7iyaE630iYyUjfsodzHgkA+seA4jH7QPLrm5JoYG56Cl0nxq5GglBtkd8AdPvR2Iodize7
QsTH3TxkcOIHKnSJWAkktV0KWGfj8S3oG3By01DoYoJQ2QAABAD4wVOEDjvj8JUEWNqjr/
uX4upFbBRU9+Y2DulBtF/wyUY1zd6ReB3fC2NSVXE1/UlW1xq5Hfx/llUMvwG7Yft6bGfU
ZAWqS7iXBUo4hczYfRY+TbMFyo0xF9lLpOtCwtk+TNUm/EdWMwBpLZSkDnJj0+y45hqYBZ
wIZgDw9aTu6Q==
---- END SSH2 ENCRYPTED PRIVATE KEY ----
```

SSH.com format



## Converting between file formats

### ssh-keygen

When generating an RSA key-pair, the OpenSSH _ssh-keygen_ program
stores the private key in the proprietary _OpenSSH Private Key_ file
format, in a file with no extension. The public key is stored in the
single-line _OpenSSH Public Key_ file format, in a file with a .pub
extension.

In most situations, the OpenSSH _ssh-keygen_ program cannot be used to
convert between private key file formats.

The only conversion it can perform is from the _SSH.com Private Key_
 format to the _PKCS #1 Private Key_ file format:

    ssh-keygen -i -f privatekey-in-ssh.com-format > privatekey.pem


The manual page is not very
clear about this: while the `-e` option can read a private key file,
it can only write a public key format. Therefore, it can only be used
to extract a public key from a private key, but it can never be used
to write out the private key in a different format.

Similarly, the `-i` option...

     -m key_format
             Specify a key format for the -i (import) or -e (export) conversion options.  The supported key formats are:
             ``RFC4716'' (RFC 4716/SSH2 public or private key), ``PKCS8'' (PEM PKCS8 public key) or ``PEM'' (PEM public key).
             The default conversion format is ``RFC4716''.  Setting a format of ``PEM'' when generating or updating a sup-
             ported private key type will cause the key to be stored in the legacy PEM private key format.




Note: the _ssh-keygen_ program cannot convert from the _OpenSSH
Private Key_ file format to the _PKCS #1 Private Key_ file
format. This is, it cannot convert from its own new format to its own
old format.

### PuTTYgen

The _PuTTYgen_ program from PuTTY can be used to extract a public key
from a PuTTY Private Key file (.ppk) -- which, despite its name,
contains both the public key and the private key.

The public key in the _SSH Public Key_ (RFC 4716) file format can be
saved to a file using the "Save public key" button.

The public key in the single-line _OpenSSH Public Key_ file format
(i.e. single line) can be copied from the "Public key for pasting into
OpenSSH authorized_keys file" text field.

